import sys, os, unittest
import ZODB
from unittest import TestCase, TestSuite, TextTestRunner, makeSuite
from Products.Event.EventService import EventService
from Products.Event.IEventService import IEventService
from Interface.Implements import objectImplements
from Products.Event.ISubscriptionAware import ISubscriptionAware
from Products.Event.ISubscriber import ISubscriber

class Subscriber:
    __implements__ = (ISubscriptionAware, ISubscriber)
    def __init__(self):
        self.notified = 0
        self.subscribed = 0
        
    def notify(self):
        self.notified = 1

    def unsubscribedFrom(self, e, type, filt):
        self.subscribed = 0

    def subscribedTo(self, e, type, filt):
        self.subscribed = 1

class TestEventService(TestCase):
    def setUp(self):
        self.e = EventService()
        self.s = Subscriber()

    def tearDown(self):
        self.e = None
        del self.e

    def testImplements(self):
        self.failUnless(IEventService in objectImplements(self.e))

    def testSubscribe(self):
        self.e.subscribe(self.s)
        self.failUnless(self.s.subscribed == 1)

    def testUnsubscribe(self):
        self.e.subscribe(self.s)
        self.e.unsubscribe(self.s)
        self.failUnless(self.s.subscribed == 0)

def test_suite():
    testsuite = makeSuite(TestEventService, 'test')
    alltests = TestSuite((testsuite,))
    return alltests

if __name__ == '__main__':
    runner = TextTestRunner(verbosity=9)
    runner.run(test_suite())
