# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: CPSSite.py,v 1.8 2003/05/10 21:37:59 fermigier Exp $

"""
  CPSSite
"""

__version__ = '$Revision: 1.8 $'[11:-2]

import Globals
from Products.CMFDefault.Portal import CMFSite
from Products.ExternalMethod.ExternalMethod import ExternalMethod

class CPSSite(CMFSite):
    meta_type = 'CPS Site'

Globals.InitializeClass(CPSSite)

manage_addCPSSiteForm = Globals.HTMLFile('zmi/manage_addCPSSiteForm', 
                                         globals())

def manage_addCPSSite(dispatcher, id,
                      title='CPS Portal',
                      description='',
                      langs_list=None,
                      root_id='root',
                      root_sn='CPS',
                      root_givenName='Root',
                      root_email='root@cps',
                      root_password1='',
                      root_password2='',
                      REQUEST=None):
    """Add a CPS Site"""

    if not root_password1:
        raise ValueError, 'You have to fill CPS Administrator password!'
    if root_password1 != root_password2:
        raise ValueError, 'Password confirmation does not match password'
    log = []
    pr = log.append

    pr('Adding a CPS Site')
    container = dispatcher.Destination()

    pr('Creating CMF Site')
    container.manage_addProduct['CMFDefault'].manage_addCMFSite(id,
                    title=title,
                    description=description,
                    create_userfolder=0)
    portal = getattr(container, id)

    pr('Creating cpsinstall External Method in CMF Site')
    cpsinstall = ExternalMethod('cpsinstall',
                                'CPS Installer',
                                'NuxCPS.cpsinstall',
                                'cpsinstall')
    portal._setObject('cpsinstall', cpsinstall)

    pr('Creating cpsupdate External Method in CMF Site')
    cpsupdate = ExternalMethod('cpsupdate',
                               'CPS Updater',
                               'NuxCPS.cpsinstall',
                               'cpsupdate')
    portal._setObject('cpsupdate', cpsupdate)

    pr('Executing cpsinstall')
    pr(portal.cpsinstall())

    pr('Executing cpsupdate')
    pr(portal.cpsupdate(langs_list=langs_list))

    pr('Configuring Portal')
    portal.portal_properties.editProperties(
        {
            'email_from_name': ('%s %s' % (root_givenName, root_sn)).strip(),
            'email_from_address': root_email,
            'smtp_server': 'localhost',
        }
    )

    pr('Creating CPS Administrator account')
    dtool = portal.portal_metadirectories.members
    dtool.createEntry(
        {
            'id_': root_id,
            'password_': root_password1,
            'roles_': ['Manager'],
            'name': root_sn,
            'givenName': root_givenName,
            'email': root_email,
        }
    )

    pr('Done')
    if REQUEST is not None:
        REQUEST.RESPONSE.setHeader('Content-Type', 'text/plain')
    return '\n'.join(log)

