# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: ContactDocument.py,v 1.12 2003/04/01 21:53:58 fermigier Exp $

"""
  Contact Document
"""

__version__ = '$Revision: 1.12 $'[11:-2]

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from BaseDocument import BaseDocument, BaseDocument_adder
from cpsdocument_utils import getImageForFile


factory_type_information = (
    {'id': 'Contact Document',
     'description': 'A Contact Document describes a person.',
     'title': '_portal_type_Contact',
     'content_icon': 'document_icon.gif',
     'product': 'NuxCPSDocuments',
     'factory': 'addContactDocument',
     'meta_type': 'Contact Document',
     'immediate_view': 'contactdocument_edit_form',
     'allow_discussion': 1,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'contactdocument_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'contactdocument_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'contactdocument_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isdocument',
                  'name': 'isdocument',
                  'action': 'isdocument',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'issearchabledocument',
                  'name': 'issearchabledocument',
                  'action': 'issearchabledocument',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


# Try to use standard LDAP names for the fields
ContactFields = (
    # basic Document fields
    'title',
    'description',
    # contact fields
    'personalTitle',
    'givenName',
    'sn',
    'mail',
    'telephoneNumber',
    'mobile',
    'facsimileTelephoneNumber',
    'homePhone',
    'jobTitle', # Not very standard. In vCard.
    'o',
    'ou',
    'roomNumber',
    'postalAddress',
    'labeledURI',
    )

_marker = []


class ContactDocument(BaseDocument):
    """Contact Document"""
    meta_type = 'Contact Document'

    security = ClassSecurityInfo()

    photo = None

    def __init__(self, id, photo='', **kw):
        BaseDocument.__init__(self, id, **kw)
        self.photo = getImageForFile(photo=photo)
        for key in ContactFields:
            value = kw.get(key, '')
            setattr(self, key, value)

    security.declarePrivate('_edit')
    def _edit(self, photo=None, photo_change=None, **kw):
        """Edit the document (private)."""
        if photo_change in ('change', None):
            if photo is not None:
                self.photo = getImageForFile(photo=photo)
        elif photo_change == 'delete':
            self.photo = None
        for key in ContactFields:
            value = kw.get(key, None)
            if value is not None:
                setattr(self, key, value)

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, **kw):
        """Edit the document."""
        self._edit(**kw)
        self.reindexObject()

    #
    # Content accessors
    #

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """Used by the catalog for basic full text indexing."""
        values = []
        for key in ContactFields:
            value = getattr(self, key, '')
            values.append(value)
        return ' '.join(values)

InitializeClass(ContactDocument)

def addContactDocument(dispatcher, id, REQUEST=None, **kw):
    """Add a Contact Document."""
    ob = ContactDocument(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)

