# Copyright (c) 2002-2003 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: URLDocument.py,v 1.10 2003/04/01 21:53:58 fermigier Exp $

"""
  URL Document
"""

__version__ = '$Revision: 1.10 $'[11:-2]

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from BaseDocument import BaseDocument, BaseDocument_adder


factory_type_information = (
    {'id': 'URL Document',
     'description': 'Une URL pointe vers un site web.',
     'title': '_portal_type_URL',
     'content_icon': 'link_icon.gif',
     'product': 'NuxCPSDocuments',
     'meta_type': 'URL Document',
     'factory': 'addURLDocument',
     'immediate_view': 'urldocument_edit_form',
     'allow_discussion': 1,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'urldocument_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'urldocument_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'urldocument_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isdocument',
                  'name': 'isdocument',
                  'action': 'isdocument',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'issearchabledocument',
                  'name': 'issearchabledocument',
                  'action': 'issearchabledocument',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


class URLDocument(BaseDocument):
    """URL Document."""
    meta_type = 'URL Document'

    security = ClassSecurityInfo()

    _properties = BaseDocument._properties + (
        {'id': 'remote_url', 'type': 'string', 'mode': 'w', 'label': 'URL'},
        )
    remote_url = ''

    #
    # Content accessors
    #
    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """Used by the catalog for basic full text indexing."""
        return '%s %s' % (self.title, self.description)

    # XXX: ???
    # def PUT
    # def manage_FTPget
    # def get_size ?

InitializeClass(URLDocument)

def addURLDocument(dispatcher, id, REQUEST=None, **kw):
    """Add a URL Document."""
    ob = URLDocument(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)

