# (c) 2002 Nuxeo SARL <http://nuxeo.com>
# (c) 2002 CIRB, Belgique
# Author: Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: CPSPortalPreferencesTool.py,v 1.8 2003/07/14 21:12:51 fermigier Exp $

"""
  PortalPreferencesTool
"""

__version__ = '$Revision: 1.8 $'[11:-2]

from zLOG import LOG, INFO

from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo

from OFS.SimpleItem import SimpleItem

from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.CMFCore.utils import UniqueObject, getToolByName

from CPSPreferences import CPSMemberPreferences


class CPSPortalPreferencesTool(UniqueObject, SimpleItem):
    """
    Portal Preferences Tool
    """

    id = 'portal_preferences'
    meta_type = 'Portal Preferences Tool'

    security = ClassSecurityInfo()

    manage_options = (
        ({'label': "Overview", 'action': 'manage_overview',},) +
        SimpleItem.manage_options
    )

    #
    # ZMI
    #
    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = DTMLFile('zmi/explainPortalPreferencesTool', globals())

    #
    # Public API
    #
    security.declarePublic('getUserPreferences')
    def getUserPreferences(self, create=1, maybe_global=1):
        """Get the user's preferences proxy"""

        mtool = getToolByName(self, 'portal_membership')
        home = mtool.getHomeFolder()
        if home is None:
            utool = getToolByName(self, 'portal_url')
            if maybe_global:
                return utool.getPortalObject().Global.Preferences
            else:
                return None

        preferences = getattr(home, 'Preferences', None)
        if preferences is not None:
            return preferences

        if not create:
            return None

        home._setObject('Preferences', CPSMemberPreferences('Preferences'))

        return home.Preferences

InitializeClass(CPSPortalPreferencesTool)
