import unittest
import Zope

import PortalSubscriptionTool

class BaseTestCase(unittest.TestCase):
    """Base setup class for the case."""

    def setUp(self):
        """Instantiate a temp tool for testing."""
        self.tool = PortalSubscriptionTool.PortalSubscriptionTool()

class FakeUser:
    def getUser(self):
        pass
    def __init__(self,name):
        self.name = name
    def getId(self):
        return self.name

class TestNotificationTool(BaseTestCase):
    """Notifiable service wrapper test class."""
    
    meta_type = "Fake Meta Type"
    def getPhysicalPath(self):
        return "/fake_physical_path"

    def testEmptyAtFirst(self):
        """Test emptiness at instanciation."""
        self.assert_(self.tool._members == ())
        self.assert_(self.tool._emails == ())

    def testAnonymousSubscription(self):
        """Test portal member registration."""
        someone = "nospam@sittingduck.net"
        self.assert_(not self.tool.isSubscribed(someone, self))
        self.tool.subscribe(someone,self)
        self.assert_(self.tool.isSubscribed(someone, self))
        self.tool.unsubscribe(someone,self)
        self.assert_(not self.tool.isSubscribed(someone, self))

    def testMemberSubscription(self):
        """Test portal member registration."""
        someone = FakeUser("Marvin Acme")
        self.assert_(not self.tool.isSubscribed(someone, self))
        self.tool.subscribe(someone,self)
        self.assert_(self.tool.isSubscribed(someone, self))
        self.tool.unsubscribe(someone,self)
        self.assert_(not self.tool.isSubscribed(someone, self))


