#
# Copyright 2001-2003 Stfane Fermigier and Nuxeo SARL
# See LICENSE.TXT for licensing information
#

import re, unittest, os.path
import Zope
app = Zope.app()
from Testing import makerequest
import Products.NuxDocument
from Products.NuxDocument import NuxDocument

# Try CMF test. OK if they fail if CMF is not installed
try:
    from Products.CMFCore.tests.base.testcase import SecurityRequestTest
    from Products.CMFCore.TypesTool import FactoryTypeInformation
    from Products.NuxDocument import CMFNuxDocument
except ImportError:
    class SecurityTest: pass


class BaseTestCase(unittest.TestCase):
    def setUp(self):
        self.app = makerequest.makerequest(Zope.app())
        get_transaction().begin()

        # Catalog for CatalogAwareness testing
        self.app.manage_addProduct['ZCatalog']\
            .manage_addZCatalog('CatalogForTests', title='')
        self.catalog = self.app.CatalogForTests

    def open(self, filename):
        directory = os.path.join(
            os.path.dirname(Products.NuxDocument.__file__), "tests")
        return open(os.path.join(directory, filename))

    def tearDown(self):
        get_transaction().abort()
        self.app._p_jar.close()


class DumbTestCase(BaseTestCase):
  def test_Dumb(self):
    title = 'XXX title YYY'
    # This should be enough to make it pass as binary data
    data = '\000\001\002qwreytroiquwe'
    self.app.manage_addProduct['NuxDocument']\
      .addZNuxDocument(id='doc', file=data, title=title)

    doc = self.app.doc
    self.assertEquals(doc.title, title)
    self.assertEquals(doc.getRaw(), data)
    self.assertEquals(doc.document_src(), data)
    self.assertEquals(doc.getHtml(), '')
    self.assertEquals(doc.PrincipiaSearchSource(), '')
    self.assertEquals(doc.text_content(), '')
    self.assertEquals(doc.MimeType(), 'application/octet-stream')
    self.assertEquals(doc.content_type(), 'application/octet-stream')

class MSWordTestCase(BaseTestCase):
  def setUp(self):
    BaseTestCase.setUp(self)
    self.data = self.open('test1.doc').read()
    self.doc = NuxDocument.NuxDocument(id='doc', file=self.data)

  def test1(self):
    pass


class ZMSWordTestCase(BaseTestCase):
  def setUp(self):
    BaseTestCase.setUp(self)
    self.data = self.open('test1.doc').read()
    self.app.manage_addProduct['NuxDocument']\
      .addZNuxDocument(id='doc', file=self.data)
    self.doc = self.app.doc

  def test_accessors(self):
    data = self.doc.document_src()
    self.assertEquals(data, self.data)

    data = self.doc.manage_FTPget()
    self.assertEquals(data, self.data)

    self.assertEquals(self.doc.getFileSize(), len(self.data))

    for data in (self.doc.PrincipiaSearchSource(), self.doc.summary()):
      self.assertEquals(data.count('Linux'), 1)
      self.assertEquals(data.count('macos'), 0)

  # XXX: this test should pass !
  #def test_catalog(self):
  #  brain_list = self.catalog(PrincipiaSearchSource='linux')
  #  self.assertEquals(len(brain_list), 1)

  #  brain_list = self.catalog(PrincipiaSearchSource='macos')
  #  self.assertEquals(len(brain_list), 0)

  def test_renderers(self):
    self.assertEquals(self.doc().count('Linux'), 1)

    #data = self.doc.document_src(REQUEST, RESPONSE)
    #REQUEST, RESPONSE = self.app.REQUEST, self.app.REQUEST.RESPONSE
    #self.assertEquals(RESPONSE.getHeader('Content-Type'), self.doc.mime_type)


class ZMSWordImagesTestCase(BaseTestCase):
  def setUp(self):
    BaseTestCase.setUp(self)
    self.data = self.open('test4.doc').read()
    self.app.manage_addProduct['NuxDocument']\
      .addZNuxDocument(id='doc', file=self.data)
    self.doc = self.app.doc

  def test_image(self):
    img = self.doc['convert0.png']
    self.assertEquals(img.get_size(), 5690)


#
# CMFDocument tests
#
class CMFMSWordTestCase1(BaseTestCase):
  "Test CMFNuxDocument with a Word document, using direct objet instantiation"

  def setUp(self):
    BaseTestCase.setUp(self)
    self.data = self.open('test1.doc').read()
    self.doc = CMFNuxDocument.CMFNuxDocument(id='doc', file=self.data)

  def test_accessors(self):
    self.assertEquals(self.doc.MimeType(), 'application/msword')
    self.assertEquals(self.doc.content_type(), 'application/msword')

    self.assertEquals(self.doc.SearchableText().count('Linux'), 1)
    self.assertEquals(self.doc.index_html().count('Linux'), 1)
    assert not re.search('<body(?i)', self.doc.CookedBody())

  def test_edit1(self):
    data = self.open('test2.doc').read()
    self.doc.edit('new title', file=data)

    self.assertEquals(self.doc.Title(), 'new title')


class CMFMSWordTestCase2(SecurityRequestTest, BaseTestCase):
  "Test CMFNuxDocument with a Word document, using CMF factory"

  def setUp(self):
    from Products.CMFCore.utils import getToolByName

    SecurityRequestTest.setUp(self)
    self.app = self.root

    # Setup CMF Site
    self.app.manage_addProduct['CMFDefault']\
      .manage_addCMFSite(id='CMFForTests')
    self.cmf = self.app.CMFForTests

    # Setup Types Tools to recognize our new class
    tt = getToolByName(self.cmf, 'portal_types')
    tt.manage_addTypeInformation(
      add_meta_type=FactoryTypeInformation.meta_type,
      id='NuxDocument', typeinfo_name='NuxDocument: CMFNuxDocument')
    self.assert_('NuxDocument' in tt.listContentTypes())

    # XXX: we should setup skins here

    # Create document by invoking factory
    self.cmf.invokeFactory('NuxDocument', id='doc', title='xxxtitleyyy')
    self.doc = self.cmf.doc
    self.data = self.open('test1.doc').read()
    self.doc._edit(title='xxxtitleyyy', file=self.data, mime_type='')

  def test_accessors(self):
    self.assertEquals(self.doc.MimeType(), 'application/msword')
    self.assertEquals(self.doc.content_type(), 'application/msword')

    # These tests only work inside a CMF
    self.assertEquals(self.doc.getIcon().count('icon_doc.gif'), 1)
    self.assertEquals(self.doc.icon().count('icon_doc.gif'), 1)

    # XXX: should work
    #self.assertEquals(self.doc.view().count('Linux'), 1)


  def test_portalCatalog(self):
    words = map(lambda x: x[0], self.cmf.portal_catalog.Vocabulary.words())
    self.assert_('linux' in words)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(DumbTestCase))
    suite.addTest(unittest.makeSuite(MSWordTestCase))
    suite.addTest(unittest.makeSuite(ZMSWordTestCase))
    suite.addTest(unittest.makeSuite(ZMSWordImagesTestCase))
    suite.addTest(unittest.makeSuite(CMFMSWordTestCase1))
    suite.addTest(unittest.makeSuite(CMFMSWordTestCase2))
    return suite
