# (c) 2002 Nuxeo SARL <http://nuxeo.com>
# (c) 2002 Florent Guillaume <mailto:fg@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: __init__.py,v 1.6 2002/09/05 13:04:33 jalon Exp $


from Products.CMFCore.WorkflowTool import WorkflowTool
from Acquisition import aq_inner, aq_parent, aq_base
from zLOG import LOG, DEBUG, INFO



if hasattr(WorkflowTool, '_nuxwf_old_doActionFor'):
    LOG('NuxExtendedWorkflow', INFO, 'Patching WorkflowTool (Refresh)')
else:
    LOG('NuxExtendedWorkflow', INFO, 'Patching WorkflowTool')
    WorkflowTool._nuxwf_old_doActionFor = WorkflowTool.doActionFor
    WorkflowTool._nuxwf_old_notifyCreated = WorkflowTool.notifyCreated
    WorkflowTool.getChainFor_NoWFA = WorkflowTool.getChainFor
    # XXX do security settings


def getWorkflowManagerFor(self, ob):
    '''
    Returns the Workflow Manager of the given object, or None.
    '''
    while not getattr(ob, '_isPortalRoot', 0):
        ob = aq_parent(aq_inner(ob))
        if ob is None:
            return None
        if getattr(ob, '_isWorkflowManager', 0):
            return ob
    return None
WorkflowTool.getWorkflowManagerFor = getWorkflowManagerFor
#security.declarePrivate('getWorkflowManagerFor') XXX


def getChainFor(self, ob):
    '''
    Returns the chain that applies to the given object.
    '''
    chain = self.getChainFor_NoWFA(ob)
    if getattr(ob, '_isWorkflowAware', 0):
        chain = ob.workflowGetChain(ob, chain, pwf=self)
    wfm = self.getWorkflowManagerFor(ob)
    if wfm is not None:
        chain = wfm.workflowManagerGetChain(ob, chain, pwf=self)
    return chain
WorkflowTool.getChainFor = getChainFor


# def listActions(self, info):
#     '''
#     Invoked by the portal_actions tool.  Allows workflows to
#     include actions to be displayed in the actions box.
#     Object actions are supplied by workflows that apply
#     to the object.  Global actions are supplied by all
#     workflows.
#     Returns the actions to be displayed to the user.
#     '''
#     actions = self._nuxwf_old_listActions(info)
#     ob = info.content
#     #if getattr(ob, '_isWorkflowAware', 0): # now done in DCWorkflow
#     #    actions = ob.workflowListObjectActions(info, actions, pwf=self)
#     wfm = self.getWorkflowManagerFor(ob)
#     if wfm is not None:
#         actions = wfm.workflowManagerListActions(info, actions, pwf=self)
#     return actions

# def listGlobalActions(self, info, actions=[], did={}):
#     '''
#     Returns the global actions.
#     '''
#     wf_ids = self.getWorkflowIds()
#     for wf_id in wf_ids:
#         if not did.has_key(wf_id):
#             wf = self.getWorkflowById(wf_id)
#             if wf is not None:
#                 a = wf.listGlobalActions(info)
#                 if a is not None:
#                     actions.extend(a)
#     return actions

def doActionFor(self, ob, action, wf_id=None, *args, **kw):
    '''
    Invoked by user interface code.
    Allows the user to request a workflow action.  The workflow object
    must perform its own security checks.
    '''
    #LOG('WorkflowTool', DEBUG, 'in doActionFor, action=%s, chain=%s, hist=%s' % (action, self.getChainFor(ob), self.workflow_history))
    wfm = self.getWorkflowManagerFor(ob)
    if wfm is not None:
        #LOG('WorkflowTool', DEBUG, ' has wfm')
        wfm.workflowManagerDoAction(ob, action, wf_id, args, kw, pwf=self)
        #LOG('WorkflowTool', DEBUG, ' not finished')
    return apply(self.doActionFor_NoWFM, (ob, action, wf_id)+args, kw)
WorkflowTool.doActionFor = doActionFor


def doActionFor_NoWFM(self, ob, action, wf_id=None, *args, **kw):
    '''
    Do a workflow action on an object.
    The workflow object must perform its own security checks.
    Do not consult the workflow manager.
    '''
    if getattr(ob, '_isWorkflowAware', 0):
        ob.workflowDoAction(action, wf_id, args, kw, pwf=self)
    return apply(self._nuxwf_old_doActionFor, (ob, action, wf_id)+args, kw)
WorkflowTool.doActionFor_NoWFM = doActionFor_NoWFM
#security.declarePrivate('doActionFor_NoWFM') XXX


def notifyCreated(self, ob):
    '''
    Notifies all applicable workflows after an object has been created
    and put in its new place.
    '''
    wfm = self.getWorkflowManagerFor(ob)
    if wfm is not None:
        finished = wfm.workflowManagerNotifyCreated(ob, pwf=self)
        if finished:
            return
    # also patch: if object already is in some workflows, don't recreate it
    if hasattr(aq_base(ob), 'workflow_history'):
        return
##     wfs = self.getWorkflowsFor(ob)
##     for wf in wfs:
##         # patch: don't recreate in already existing workflows
##         if self.getHistoryOf(wf.getId(), ob) is None:
##             wf.notifyCreated(ob)
    return self._nuxwf_old_notifyCreated(ob)
WorkflowTool.notifyCreated = notifyCreated



# patch existing class

#security = WorkflowTool.security

## WorkflowTool._nuxwf_old_listActions = WorkflowTool.listActions
## WorkflowTool.listActions = listActions

## #security.declarePrivate('listGlobalActions')
## WorkflowTool.listGlobalActions = listGlobalActions



from Products.CMFCore import WorkflowTool 
WorkflowTool.AUTO_MIGRATE_WORKFLOW_TOOLS = 0
