# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: DocFilterBox.py,v 1.12 2003/04/06 09:55:52 fermigier Exp $

"""
  DocFilterBox
"""

__version__='$Revision: 1.12 $'[11:-2]

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent


from BaseBox import BaseBox


def addDocFilterBox(dispatcher, id, REQUEST=None, **kw):
    """Add a Doc Filter Box."""
    ob = DocFilterBox(id, **kw)
    dispatcher._setObject(id, ob)
    if REQUEST is not None:
        url = dispatcher.DestinationURL()
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)


factory_type_information = (
    {'id': 'Doc Filter Box',
     'title': '_portal_type_Doc Filter Box',
     'description': ('A Doc Filter Box lists documents.'),
     'meta_type': 'Doc Filter Box',
     'content_icon': 'box_icon.gif',
     'product': 'NuxPortal',
     'factory': 'addDocFilterBox',
     'immediate_view': 'docfilterbox_edit_form',
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': 'View',
                  'action': 'basebox_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': 'Edit',
                  'action': 'docfilterbox_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'render_title',
                  'name': 'Render title',
                  'action': 'basebox_render_title',
                  'visible': 0,
                  'permissions': (View,)},
                 {'id': 'render_body',
                  'name': 'Render body',
                  'action': 'docfilterbox_render_body',
                  'visible': 0,
                  'permissions': (View,)},
                 {'id': 'isportalbox',
                  'name': 'isportalbox',
                  'action': 'isportalbox',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


class DocFilterBox(BaseBox):
    """
    A Doc Filter Box filters documents.
    """
    meta_type = 'Doc Filter Box'
    # XXX Hack for CMF 1.3
    portal_type = 'Doc Filter Box'

    security = ClassSecurityInfo()

    _properties = BaseBox._properties + (
        {'id': 'title', 'type': 'string', 'mode': 'w',
            'label': 'Title'},
        {'id': 'description', 'type': 'text', 'mode': 'w',
            'label': 'Description'},
        {'id': 'types', 'type': 'lines', 'mode': 'w',
            'label': 'Document types'},
        {'id': 'document_state', 'type': 'string', 'mode': 'w',
            'label': 'Document state'},
        {'id': 'max_num', 'type': 'int', 'mode': 'w',
            'label': 'Maximum number'},
        {'id': 'display_type', 'type': 'string', 'mode': 'w',
            'label': 'Display type'},
        {'id': 'sort_by', 'type': 'string', 'mode': 'w',
            'label': 'Sort by'},
        {'id': 'sort_desc', 'type': 'boolean', 'mode': 'w',
            'label': 'Reverse sort'},
        {'id': 'contextual', 'type': 'boolean', 'mode': 'w',
            'label': 'Contextual'},
        )

    contextual = 0

    def __init__(self, id,
                 title='',
                 description='',
                 types=[],
                 document_state='',
                 max_num=0,
                 display_type='',
                 sort_by='',
                 sort_desc=0,
                 contextual=0,
                 **kw):
        apply(BaseBox.__init__, (self, id), kw)
        self.title = title
        self.description = description
        self.types = types
        self.document_state = document_state
        self.max_num = max_num
        self.display_type = display_type
        self.sort_by = sort_by
        self.sort_desc = sort_desc
        self.contextual = contextual

InitializeClass(DocFilterBox)

