# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: WGNavBox.py,v 1.9 2003/04/06 09:55:52 fermigier Exp $

"""
  WGNavBox
"""

__version__='$Revision: 1.9 $'[11:-2]

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent


from BaseBox import BaseBox


def addWGNavBox(dispatcher, id, REQUEST=None, **kw):
    """Add a WGNav Box."""
    ob = WGNavBox(id, **kw)
    dispatcher._setObject(id, ob)
    if REQUEST is not None:
        url = dispatcher.DestinationURL()
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)


factory_type_information = (
    {'id': 'WGNav Box',
     'title': '_portal_type_WGNav Box',
     'description': ('A WGNav Box is for navigation in workgroups.'),
     'meta_type': 'WGNav Box',
     'content_icon': 'box_icon.gif',
     'product': 'NuxPortal',
     'factory': 'addWGNavBox',
     'immediate_view': 'wgnavbox_edit_form',
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': 'View',
                  'action': 'basebox_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': 'Edit',
                  'action': 'wgnavbox_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'render_title',
                  'name': 'Render title',
                  'action': 'basebox_render_title',
                  'visible': 0,
                  'permissions': (View,)},
                 {'id': 'render_body',
                  'name': 'Render body',
                  'action': 'wgnavbox_render_body',
                  'visible': 0,
                  'permissions': (View,)},
                 {'id': 'isportalbox',
                  'name': 'isportalbox',
                  'action': 'isportalbox',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


class WGNavBox(BaseBox):
    """
    A WGNavBox renders a workgroup hierarchy for navigation.
    """
    meta_type = 'WGNav Box'
    # XXX Hack for CMF 1.3
    portal_type = 'WGNav Box'

    security = ClassSecurityInfo()

    _properties = BaseBox._properties + (
        {'id': 'title', 'type': 'string', 'mode': 'w',
            'label': 'Title'},
        {'id': 'description', 'type': 'string', 'mode': 'w',
            'label': 'Description'},
        {'id': 'paths', 'type': 'lines', 'mode': 'w',
            'label': 'Paths'},
        {'id': 'depth', 'type': 'int', 'mode': 'w',
            'label': 'Depth'},
        )

    def __init__(self, id,
                 title='',
                 description='',
                 paths=[],
                 depth=0,
                 **kw):
        apply(BaseBox.__init__, (self, id), kw)
        self.title = title
        self.description = description
        self.paths = paths
        self.depth = depth


InitializeClass(WGNavBox)

